#ifndef APPLICATION_H
#define APPLICATION_H


class Benchmarker;
class ParticleGroup;
class Renderer;
#include "ExampleApplication.h"
#include "KeyListener.h"
#include "Material.h"

namespace Ogre
{
	class Entity;
	class Light;
	class SceneNode;
}
#include "OgreFrameListener.h"
#include "OgreVector3.h"
#include <string>
#include <vector>


class Application: public ExampleApplication, KeyListener, Ogre::FrameListener
{
public:
	Application();
	virtual ~Application();

	virtual void createScene();
	virtual void createFrameListener();

	virtual void keyPressed(const OIS::KeyEvent& e);
	virtual bool frameStarted(const Ogre::FrameEvent& evt);
	virtual bool frameEnded(const Ogre::FrameEvent& evt);

	void setOverlayTexts(const std::string& text0, const std::string& text1) const;
	void setParticleSprayActive(bool active);
	unsigned int getNumMaterials() const;
	void setCurrentMaterialIndex(unsigned int index);
	unsigned int getCurrentMaterialIndex() const;
	std::string getCurrentMaterialName() const;
	int getVersionNumber() const;
private:
	void switchParticleEmittersActive();
	void setVolumeDensity(float density);
	void updateKeysDown(float time);
	void updateCameraAnimation(float time);
	void updateOverlayText() const;
	void updateFullInformation(std::stringstream& text0, std::stringstream& text1) const;
	void updateSmallInformation(std::stringstream& text0) const;

	enum CameraMode
	{
		CM_AUTOMATIC,
		CM_FREE
	};
	CameraMode mCameraMode;
	float mCameraAnimationPhase;
	float mCameraAnimationSpeed;

	enum InfoState
	{
		IS_SMALL,
		IS_FULL,
		IS_NONE
	};
	InfoState mInfoState;

	Ogre::SceneNode* mParticleRootNode;
	ParticleGroup* mParticleGroup;
	bool mParticleEmittersActive;
	std::vector<Material> materials;
	unsigned int currentMaterial;
	float mVolumeDensity;

	Renderer* mRenderer;
	Ogre::SceneNode* mEnvironmentNode;
	Ogre::Entity* mEnvironmentEntity;
	Ogre::Light* mLight0;
	Ogre::Light* mLight1;

	Benchmarker* mBenchmarker;
};


#endif