#include "ExampleApplication.h"
#include "ExampleFrameListener.h"
#include "InputManager.h"

#include "OgreCamera.h"
#include "OgreConfigFile.h"
#include "OgreRenderWindow.h"
#include "OgreRoot.h"
#include "OgreSceneManager.h"
#include "OgreTextureManager.h"


ExampleApplication::ExampleApplication():
	mFrameListener(NULL),
	mRoot(NULL)
{
}


ExampleApplication::~ExampleApplication()
{
    delete mFrameListener;
    delete mRoot;
}


void ExampleApplication::go()
{
    if (!setup())
	{
        return;
	}

    mRoot->startRendering();

    // clean up
    destroyScene();
}


bool ExampleApplication::setup(void)
{
    mRoot = new Ogre::Root();

    setupResources();

    if (!configure())
	{
		return false;
	}

    chooseSceneManager();
    createCamera();
    createViewports();

    // Set default mipmap level (NB some APIs ignore this)
    Ogre::TextureManager::getSingleton().setDefaultNumMipmaps(5);

	// Create any resource listeners (for loading screens)
	createResourceListener();
	// Load resources
	loadResources();

	//initialize input
	InputManager::get();

	// Create the scene
    createScene();

	createFrameListener();

    return true;
}


bool ExampleApplication::configure(void)
{
    // Show the configuration dialog and initialise the system
    // You can skip this and use root.restoreConfig() to load configuration
    // settings if you were sure there are valid ones saved in ogre.cfg
    if (mRoot->showConfigDialog())
    {
        // If returned true, user clicked OK so initialise
        // Here we choose to let the system create a default rendering window by passing 'true'
        mWindow = mRoot->initialise(true, "Volumetrics Demo");
        return true;
    }
    else
    {
        return false;
    }
}


void ExampleApplication::chooseSceneManager()
{
    // Create the SceneManager, in this case a generic one
    mSceneMgr = mRoot->createSceneManager(Ogre::ST_GENERIC, "ExampleSMInstance");
}


void ExampleApplication::createCamera()
{
    // Create the camera
    mCamera = mSceneMgr->createCamera("PlayerCam");

    // Position it at 500 in Z direction
    mCamera->setPosition(Ogre::Vector3(0, 0, 5));

    // Look back along -Z
    mCamera->lookAt(Ogre::Vector3(0, 0, -3));
    mCamera->setNearClipDistance(0.05);
}


void ExampleApplication::destroyScene()
{
}


void ExampleApplication::createViewports()
{
    // Create one viewport, entire window
	mViewport = mWindow->addViewport(mCamera);
    mViewport->setBackgroundColour(Ogre::ColourValue(0,0,0));

    // Alter the camera aspect ratio to match the viewport
    mCamera->setAspectRatio(Ogre::Real(mViewport->getActualWidth()) / Ogre::Real(mViewport->getActualHeight()));
}


void ExampleApplication::setupResources()
{
    // Load resource paths from config file
    Ogre::ConfigFile cf;
    cf.load("resources.cfg");

    // Go through all sections & settings in the file
    Ogre::ConfigFile::SectionIterator seci = cf.getSectionIterator();

    Ogre::String secName, typeName, archName;
    while (seci.hasMoreElements())
    {
        secName = seci.peekNextKey();
        Ogre::ConfigFile::SettingsMultiMap *settings = seci.getNext();
        Ogre::ConfigFile::SettingsMultiMap::iterator i;
        for (i = settings->begin(); i != settings->end(); ++i)
        {
            typeName = i->first;
            archName = i->second;
            Ogre::ResourceGroupManager::getSingleton().addResourceLocation(
                archName, typeName, secName);
        }
    }
}


void ExampleApplication::createResourceListener()
{
}


void ExampleApplication::loadResources()
{
	// Initialise, parse scripts etc
	Ogre::ResourceGroupManager::getSingleton().initialiseAllResourceGroups();
}