/*
-----------------------------------------------------------------------------
This source file is part of OGRE
    (Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2005 The OGRE Team
Also see acknowledgements in Readme.html

You may use this sample code for anything you like, it is not covered by the
LGPL like the rest of the engine.
-----------------------------------------------------------------------------
*/
/*
-----------------------------------------------------------------------------
Filename:    ExampleFrameListener.h
Description: Defines an example frame listener which responds to frame events.
This frame listener just moves a specified camera around based on
keyboard and mouse movements.
Mouse:    Freelook
W or Up:  Forward
S or Down:Backward
A:        Step left
D:        Step right
             PgUp:     Move upwards
             PgDown:   Move downwards
             F:        Toggle frame rate stats on/off
			 R:        Render mode
             T:        Cycle texture filtering
                       Bilinear, Trilinear, Anisotropic(8)
             P:        Toggle on/off display of camera position / orientation
-----------------------------------------------------------------------------
*/

#ifndef __ExampleFrameListener_H__
#define __ExampleFrameListener_H__


namespace Ogre
{
	class Camera;
	class Overlay;
}
#include "OgreFrameListener.h"
#include "OgreMaterialManager.h"
#include "OgreMath.h"
#include "OgreVector3.h"


class ExampleFrameListener: public Ogre::FrameListener
{
public:
    // Constructor takes a RenderWindow because it uses that to determine input context
	ExampleFrameListener(Ogre::RenderWindow* win, Ogre::Camera* cam);

    virtual ~ExampleFrameListener();
    virtual bool processUnbufferedKeyInput(const Ogre::FrameEvent& evt);
    bool processUnbufferedMouseInput(const Ogre::FrameEvent& evt);
	void moveCamera();
    void showDebugOverlay(bool show);

    // Override frameStarted event to process that (don't care about frameEnded)
    virtual bool frameStarted(const Ogre::FrameEvent& evt);
    virtual bool frameEnded(const Ogre::FrameEvent& evt);

	void setCameraRotationAllowed(bool allowed);

protected:
	void updateStats();

	int mSceneDetailIndex;
    Ogre::Real mMoveSpeed;
    Ogre::Degree mRotateSpeed;
    Ogre::Overlay* mDebugOverlay;

    Ogre::Camera* mCamera;

    Ogre::Vector3 mTranslateVector;
    Ogre::RenderWindow* mWindow;
    bool mStatsOn;
	unsigned int mNumScreenShots;
    float mMoveScale;
    Ogre::Degree mRotScale;
    // just to stop toggles flipping too fast
	Ogre::Real mTimeUntilNextToggle ;
    Ogre::Radian mRotX, mRotY;
    Ogre::TextureFilterOptions mFiltering;
    int mAniso;

	bool mCameraRotationAllowed;
};


#endif