#ifndef INPUTMANAGER_H
#define INPUTMANAGER_H


class KeyListener;
class MouseListener;

namespace OIS
{
	class InputManager;
}
#include "OISKeyboard.h"
#include "OISMouse.h"
#include <set>


class InputManager: public OIS::KeyListener, OIS::MouseListener
{
public:
	enum MouseButton
	{
		MB_LEFT,
		MB_MIDDLE,
		MB_RIGHT
	};
	static InputManager* get();

	void capture();

	bool isKeyDown(OIS::KeyCode keyCode) const;
	bool isMouseButtonDown(MouseButton mouseButton) const;
	int getMouseRelativeX();
	int getMouseRelativeY();

	void addKeyListener(::KeyListener* keyListener);
	void removeKeyListener(::KeyListener* keyListener);
	void clearAllKeyListeners();
	void setExclusiveKeyListener(::KeyListener* keyListener);
	void removeExclusiveKeyListener();

	void addMouseListener(::MouseListener* mouseListener);
	void removeMouseListener(::MouseListener* mouseListener);
	void clearAllMouseListeners();
	void setExclusiveMouseListener(::MouseListener* mouseListener);
	void removeExclusiveMouseListener();

	virtual bool keyPressed(const OIS::KeyEvent& e);
	virtual bool keyReleased(const OIS::KeyEvent& e);

	virtual bool mouseMoved(const OIS::MouseEvent& e);
	virtual bool mousePressed(const OIS::MouseEvent& e, OIS::MouseButtonID id);
	virtual bool mouseReleased(const OIS::MouseEvent& e, OIS::MouseButtonID id);
private:
	InputManager();
	~InputManager();

	OIS::InputManager*	inputManager;
	OIS::Keyboard*		keyboard;
	OIS::Mouse*			mouse;


	std::set<::KeyListener*> keyListeners;
	::KeyListener* exclusiveKeyListener;
	std::set<::MouseListener*> mouseListeners;
	::MouseListener* exclusiveMouseListener;
};


#endif