#ifndef PARTICLEGROUP_H
#define PARTICLEGROUP_H


class MeshParticle;
class Renderer;

namespace Ogre
{
	class SceneNode;
	class Vector3;
}
#include <string>
#include <vector>


class ParticleGroup
{
public:
	ParticleGroup(Ogre::SceneNode* particleRootNode,
				  const std::string& mesh,
				  const std::string& material,
				  bool materialRequiresDepthTexture,
				  bool emittersActive,
				  float volumeDensity,
				  Renderer* renderer);

	~ParticleGroup();

	void update(float time, const Ogre::Vector3& viewPosition);

	void setVolumeMode(const std::string& mesh,
					   const std::string& material,
					   bool materialRequiresDepthTexture);

	void setEmittersActive(bool active);
	void setVolumeDensity(float density);
private:
	void updateParticleEmitting(float time);
	void createParticle(const Ogre::Vector3& position,
						const Ogre::Vector3& speed,
						float lifeTime,
						float scale,
						bool animateParemeters);

	Renderer* mRenderer;

	Ogre::SceneNode* mGroupRoot;

	std::vector<MeshParticle*> mParticles;

	std::string mMesh, mMaterial;
	bool mMaterialRequiresDepthTexture;

	bool mEmittersActive;
	float mTimeUntilNextEmitting;
	float mTimeBetweenParticleEmitting;

	float mVolumeDensity;

	static std::vector<std::pair<std::string, std::string> > sTextures;
};


#endif